"use strict";
this.name = "StationDockControl_MFD";
this.author = "phkb";
this.copyright = "2015 phkb";
this.description = "Displays the list of ships docked at the current station in an MFD.";
this.licence = "CC BY-NC-SA 3.0";

this._mfdLineLength = 14.2; // horizontal measure of the max width of a line in the MFD (15 is the width of an MFD, minus some margin)
this._mfdMaxLines = 10; // maximum number of lines visible in the MFD (10 - 1 for the header line)
this._shipSpace = 13; // column width for ship name (remaining space will be departure time)
this._update = null; // object to hold timer that will refresh the MFD
this._refreshMFDTime = 11; // number of seconds between MFD view refreshes
this._stationTarget = null; // the current target for the MFD
this._playerIsDocking = false; // flag to indicate the player has requested docking clearance
this._dockTimer = null;
this._mfdID = -1;
this._debug = false;

//=============================================================================================================
// system functions

//-------------------------------------------------------------------------------------------------------------
this.startUpComplete = function () {
	// if the OXP is disabled, then delete the relevant functions here
	if (worldScripts.StationDockControl._disable === true) {
		delete this.shipTargetAcquired;
		delete this.playerRequestedDockingClearance;
		delete this.shipWillLaunchFromStation;
		delete this.shipExitedWitchspace;
		delete this.startUpComplete;
		delete this.shipDockedWithStation;
		delete this.playerWillEnterWithspace;
		delete this.playerStartedAutoPilot;
		delete this.playerCancelledAutoPilot;
	} else {
		this._debug = worldScripts.StationDockControl._debug;

		var h = worldScripts.hudselector;
		if (h) h.$HUDSelectorAddMFD(this.name, "LaunchQueueMFD");
	}
}

//-------------------------------------------------------------------------------------------------------------
// turn on mfd when a station is targeted
this.shipTargetAcquired = function (target) {
	if (target.isStation) {
		// if the control is enabled and we don't have a station target set yet...
		//if (!this._stationTarget) { // removed this condition, to cater for the (very rare) situation of having two stations in range of the player.
		// if the station has NPC traffic and is publishing a view of docked ships...
		//if (target.hasNPCTraffic && worldScripts.StationDockControl_Interface.$checkStationRoleForView(target)) {
		// only show the MFD for galcop aligned stations
		if (target.hasNPCTraffic && target.allegiance === "galcop") {
			// set the target station and start running the update timer
			this._stationTarget = target;
			if (!this._update || this._update.isRunning === false) {
				this._update = new Timer(this, this.$mfd_updateview, 1, this._refreshMFDTime);
			}
		}
		// }
	}
}

//-------------------------------------------------------------------------------------------------------------
this.shipWillLaunchFromStation = function (station) {
	this._playerIsDocking = false;
	this.$mfd_updateview();
}

//-------------------------------------------------------------------------------------------------------------
this.playerRequestedDockingClearance = function (message) {
	if (this._debug) log(this.name, "Player docking request message: " + message);
	if (this._stationTarget) {
		if (message === "DOCKING_CLEARANCE_GRANTED" || message === "DOCKING_CLEARANCE_EXTENDED" || message === "DOCKING_CLEARANCE_DENIED_TRAFFIC_INBOUND" || message === "DOCKING_CLEARANCE_DENIED_TRAFFIC_OUTBOUND") {
			this._playerIsDocking = true;
			if (message === "DOCKING_CLEARANCE_GRANTED" || message === "DOCKING_CLEARANCE_EXTENDED") {
				// start a timer so we can track when the clearance expires
				this.$stopTimers();
				this._dockTimer = new Timer(this, this.$dockingExpired, 121, 0);
			}
		} else {
			this._playerIsDocking = false;
			this.$stopTimers();
		}
		this.$mfd_updateview();
	}
}

//-------------------------------------------------------------------------------------------------------------
this.playerStartedAutoPilot = function () {
	this._playerIsDocking = true;
	this.$mfd_updateview();
}

//-------------------------------------------------------------------------------------------------------------
this.playerCancelledAutoPilot = function () {
	this._playerIsDocking = false;
	this.$mfd_updateview();
}

//-------------------------------------------------------------------------------------------------------------
this.shipDockedWithStation = function (station) {
	this.$stopTimers();
}

//-------------------------------------------------------------------------------------------------------------
this.playerWillEnterWitchspace = function () {
	this.$stopTimers();
}

//-------------------------------------------------------------------------------------------------------------
this.shipExitedWitchspace = function () {
	this._stationTarget = null;
	this.$mfd_updateview();
}

//=============================================================================================================
// mfd functions

//-------------------------------------------------------------------------------------------------------------
// stops the dock timer
this.$stopTimers = function $stopTimers() {
	if (this._dockTimer && this._dockTimer.isRunning) {
		this._dockTimer.stop();
		delete this._dockTimer;
	}
}

//-------------------------------------------------------------------------------------------------------------
// checks for an expired docking request
this.$dockingExpired = function $dockingExpired() {
	if (this._playerIsDocking === true) {
		this._playerIsDocking = false;
		this.$mfd_updateview();
	}
}

//-------------------------------------------------------------------------------------------------------------
// updates the text in the MFD
this.$mfd_updateview = function $mfd_updateview() {
	var p = player.ship;
	if (p.isValid === false) return;

	this.$findMFDID();

	if (this._stationTarget && this._stationTarget.isValid && p.position.distanceTo(this._stationTarget) < p.scannerRange) {
		if (p.isInSpace) {
			var w = worldScripts.StationDockControl_Interface;
			var sdc = worldScripts.StationDockControl;
			var output = "";
			var lineCount = 1; // header line
			// if a repopulate is in progress, don't try and show data
			if (sdc.$stationIsRepopulating(this._stationTarget) === false) {
				var docklist = w.$getStationDockList(this._stationTarget);
				// find dock for station
				if (this._stationTarget.subEntities) {
					for (var i = 0; i < this._stationTarget.subEntities.length; i++) {
						if (this._stationTarget.subEntities[i].isDock) {
							// get count on number of ships docking
							// note: this seems to get out of sync with the count of ships returned by the $getDockingShips function.
							// Need to find out why
							var docking = 0;
							docking = this._stationTarget.subEntities[i].dockingQueueLength;
							// the player is not counted in dockingQueueLength, so add them in here, if they're docking
							if (this._playerIsDocking) docking += 1;
							// start building the output
							var pt1 = "Ships docking: " + docking;
							var pt2 = "In launch corridor: " + this._stationTarget.subEntities[i].launchingQueueLength;
							// build the top line so part 1 is on the left and part 2 is on the right, with all the extra space in between
							if (defaultFont.measureString(pt1 + pt2) < this._mfdLineLength) {
								output = w.$padTextRight(pt2 + pt1, this._mfdLineLength);
								output = output.substring(pt2.length) + pt2 + "\n";
							} else {
								// if it's already too wide width any spacing, just concat the text and let the MFD handle the squashing
								output = pt1 + " " + pt2 + "\n";
							}

							// if there's ships docking, get the details here
							if (docking > 0) {
								var data = this.$getDockingShips(this._stationTarget);
								output += data.text;
								lineCount += data.lines;
							}
							break;
						}
					}
				}
				// if we haven't filled up the MFD with inbound ships, then fill the rest with docked ships
				var rsn = sdc._rsnInstalled;
				if (rsn === true && w._rsnInMFD === false) rsn = false;
				if (lineCount <= this._mfdMaxLines) {
					for (var i = 0; i < docklist.length; i++) {
						var offset = 0;
						if (docklist[i].departureTime < 1) offset = 2;

						output += w.$padTextRight(docklist[i].shipType + (rsn === true ? ": " + docklist[i].shipName : ""), this._shipSpace - offset) + " " + w.$departureTimeFormat(docklist[i].departureTime) + "\n";
						lineCount += 1;
						// break out once we reach the limit
						if (lineCount === this._mfdMaxLines) break;
					}
				}
			} else {
				output += "Establishing communications link\n";
				output += "with station traffic control.\n";
				output += "Please stand by.";
			}
			// set the text in the MFD
			p.setMultiFunctionText("LaunchQueueMFD", output, false);
			// check if the last place to have the launch queue mfd is blank (that is, unset), and if so, set it back to be the launch queue mfd
			if (this._mfdID >= 0 && p.multiFunctionDisplayList[this._mfdID] === "") {
				p.setMultiFunctionDisplay(this._mfdID, "LaunchQueueMFD");
			}
		}
	} else {
		// out of range, so display some appropriate text
		p.setMultiFunctionText("LaunchQueueMFD", "Traffic Control\n\nNo station in range", false);
		if (this._mfdID >= 0) p.setMultiFunctionDisplay(this._mfdID, "");

		if (this._update && this._update.isRunning) this._update.stop();
		delete this._update;
		this._stationTarget = null;
	}
}

//-------------------------------------------------------------------------------------------------------------
// gets list of ships who are currently trying to dock with the station and returns formatted output for mfd
this.$getDockingShips = function $getDockingShips(station) {
	function $isDocking(entity) {
		return entity.isShip && !entity.isPlayer && entity.isValid && entity.isPiloted && !entity.isStation && entity.dockingInstructions && entity.dockingInstructions.station === station;
	}
	function compare(a, b) {
		if (a.shipClassName < b.shipClassName) return -1;
		if (a.shipClassName > b.shipClassName) return 1;
		return 0;
	}

	var ships = system.filteredEntities(this, $isDocking, station, station.scannerRange);
	var output = "";
	var w = worldScripts.StationDockControl_Interface;
	var sdc = worldScripts.StationDockControl;
	var rsn = sdc._rsnInstalled;
	if (!w._rsnInMFD) rsn = false;
	var lineCount = 0;

	// if the player is trying to dock, put them first on the list, so they will always be visible
	if (this._playerIsDocking) {
		ships.push(player.ship);
	}

	// sort the list by ship class name. We don't know (and can't know) the order in which ships dock, so a classname sort order will do
	ships.sort(compare);

	// build the output
	for (var i = 0; i < ships.length; i++) {
		output += w.$padTextRight((rsn == false ? ships[i].shipClassName : ships[i].displayName), this._shipSpace) + " Inbound\n";
		lineCount += 1;
		if (lineCount === this._mfdMaxLines) break;
	}

	return {
		text: output,
		lines: lineCount
	};
}

//-------------------------------------------------------------------------------------------------------------
// records the index of the MFD that currently holds the traffic control mfd
this.$findMFDID = function $findMFDID() {
	var p = player.ship;
	if (p && p.multiFunctionDisplayList) {
		for (var i = 0; i < p.multiFunctionDisplayList.length; i++) {
			if (p.multiFunctionDisplayList[i] === "LaunchQueueMFD") this._mfdID = i;
		}
	}
}